==> weather-2.01.tar.bz2: weather/weather_parser.cpp <==
/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "weather_parser.h"

WeatherParser::WeatherParser(QString& page, ConfigFile* cf)
:
	strPage(page),
	config(cf)
{
}

/**
	Formatowanie wyjciowej prognozy pogody
**/
void WeatherParser::getData(ConfigFile* wConfig, Forecast& forecast, const QString& iconPath)
{
	bool cs = wConfig->readBoolEntry("Default","CaseSensitive");
	
	int num_layouts = wConfig->readNumEntry("Default","Layouts");
	WDataValue dValue;
	
	QString actualName;
	QValueList<WDataValue> valsList;
	QStringList nextDaysNamesList;
	QValueList<WDataValue> nextDaysValsList;
	
	int cursor = 0;
	for(int i=0; i<num_layouts; i++)
	{
		QString layoutSection = QString("Layout%1").arg(i+1);
		QString layoutName = wConfig->readEntry(layoutSection,"Name");
		int num_repeats = wConfig->readNumEntry(layoutSection,"Repeats");
		int num_values = wConfig->readNumEntry(layoutSection,"Values");
		
		for(int j=0; j<num_repeats; j++)
		{
			for(int k=0; k<num_values; k++)
			{
				QString valSection = QString("Layout%1Value%2").arg(i+1).arg(k+1);
				
				dValue.Name = wConfig->readEntry(valSection,"Name");
				dValue.Start = wConfig->readEntry(valSection,"Start");
				dValue.End = wConfig->readEntry(valSection,"End");
				dValue.Content = "";
				
				getDataValue(dValue,cursor, wConfig, cs);
				
				if(dValue.Content.isEmpty())
					continue;
				else if(dValue.Name == "Phantom")
					continue;
				
				if(layoutName == "Name")
					forecast.LocationName = dValue.Content;
				else if(layoutName == "Actual")
				{
					if(dValue.Name == "Name")
						actualName = dValue.Content;
					else
						valsList.push_back(dValue);
				}
				else if(layoutName == "DaysNames")
				{
					if(dValue.Name == "Actual")
						actualName = dValue.Content;
					else if(dValue.Name == "NextDays")
						nextDaysNamesList.push_back(dValue.Content);
				}
				else if(layoutName == "NextDays" && dValue.Name == "Name")
					nextDaysNamesList.push_back(dValue.Content);
				else
					nextDaysValsList.push_back(dValue);
			}
		}
	}
	
	QString Data("");
	
	ForecastDay fDay;
	
	fDay["Name"] = actualName;
	for(QValueList<WDataValue>::iterator it = valsList.begin(); it != valsList.end(); it++)
	{
		if((*it).Name == "Icon")
			fDay["Icon"] = iconPath + wConfig->readEntry("Icons",(*it).Content);
		else
			fDay[(*it).Name] = (*it).Content;
	}
	
	forecast.Days.push_back(fDay);
	fDay.clear();
	
	int num_vals = nextDaysValsList.count();
	int num_names = nextDaysNamesList.count();
	int T = num_vals/num_names;
	
	Data = "";
	for(int i=0; i<num_vals; i++)
	{
		WDataValue& val = nextDaysValsList[i];
	
		if(i%T == 0)
			fDay["Name"] = nextDaysNamesList[i/T];
		
		if(val.Name == "Icon")
			fDay["Icon"] = iconPath + wConfig->readEntry("Icons",val.Content);
		else
			fDay[val.Name] = val.Content;
		
		if(i%T == T-1)
		{
			forecast.Days.push_back(fDay);
			fDay.clear();
			
			Data = "";
		}
	}
}

/**
	Wyawianie fragmentu ze strony
**/
void WeatherParser::getDataValue(WDataValue& wdata, int& cursor, ConfigFile* wConfig, bool CaseSensitive)
{
	long int start, end;
	long int startData;
	
	start = strPage.find(wdata.Start, cursor, CaseSensitive);
	if(start == -1)
		return;
	
	startData = start + wdata.Start.length();
	
	end = strPage.find(wdata.End, startData, CaseSensitive);
	if(end == -1)
		return;
	
	cursor = end;
	wdata.Content = tagClean(strPage.mid(startData, end-startData));
}

/**
	Zwraca wyszukane identyfikatory miast dla szukanego miasta
**/
void WeatherParser::getSearch(QStringList& names, QStringList& codes, ConfigFile* wConfig)
{
	bool CaseSensitive = wConfig->readBoolEntry("Default","CaseSensitive");

	long int start, separator, end, current;
	long int startData, sepData;
	bool idFirst;
	QString section, starttag, septag, endtag, first, second;
	
	int countResults = wConfig->readNumEntry("Name Search","SearchResults");
	
	for(int i=0; i<countResults; i++)
	{
		section = QString("SearchResult%1").arg(i+1);
		idFirst = wConfig->readBoolEntry(section,"IDFirst");
		starttag = wConfig->readEntry(section,"Start");
		septag = wConfig->readEntry(section,"Separator");
		endtag = wConfig->readEntry(section,"End");
	
		current = 0;
		do{
			start = strPage.find(starttag, current, CaseSensitive);	
			startData = start + starttag.length();
			separator = strPage.find(septag, startData, CaseSensitive);
			sepData = separator + septag.length();
			end = strPage.find(endtag, sepData, CaseSensitive);
			
			if(end != -1 && start != -1 && separator != -1)
			{
				first = strPage.mid(startData, separator-startData);
				second = strPage.mid(sepData, end-sepData);
				
				if(!first.isEmpty() && !second.isEmpty())
				{
					if(idFirst)
					{
						names.push_front(tagClean(second));
						codes.push_front(first);
					}
					else
					{
						names.push_front(tagClean(first));
						codes.push_front(second);
					}
				}
				current = end+endtag.length();
			}
		}while(end != -1 && start != -1 && separator != -1);
	}
}

/**
	Wyowienie szukanego ID miasta z adresu URL.
	(niektre serwisy przekierowywuj od razu na
	stron z prognoz, gdy znajd tylko jedno miasto)
**/
QString WeatherParser::getFastSearch(const QString& link, const ConfigFile& wConfig)
{
	QString starttag, endtag;
	long int start, end, startData;
	
	starttag = wConfig.readEntry("Name Search","FastSearch Start");
	endtag = wConfig.readEntry("Name Search","FastSearch End");
	
	start = link.find(starttag, 0, false);
	startData = start + starttag.length();
	
	if(endtag.isEmpty())
		end = link.length();
	else
		end = link.find(endtag, startData, false);
	
	if(start == -1 || end == -1)
		return "";
	else
		return link.mid(startData, end-startData);
}

/**
	Czyszczenie bufora z tagw HTML i zamienianie znakw specjalnych
**/
QString WeatherParser::tagClean(QString str)
{
	//str.replace("&deg;","");
	str.replace("&nbsp;"," ");
	
	int start, end;
	start = 0;
	do{
		start = str.find("<",start);
		end = str.find(">",start+1);
		
		if(start != -1 && end != -1)
			str.replace(start,end+1-start, " ");
			
	}while(start != -1 && end != -1);
	
	str.replace("\n"," ");
	str.replace("\r"," ");
	str.replace("  "," ");
	str.replace(" ,",",");
	str.replace(" .",".");
	str.replace(" :",":");
	str.replace(" / ","/");
	return str;
}


/**
	ForecastContener
**/

bool ForecastContener::getForecast(const QString& configFile, const QString& location, Forecast& forecast, int hours)
{
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); it++)
	{
		if( (*it).LocationID == location && (*it).config == configFile)
		{
			if( (*it).loadTime.elapsed() > hours*60*60*1000 )
			{
				list.remove(it);
				return false;
			}
			else
			{
				forecast = (*it);
				return true;
			}
		}
	}
	
	return false;
}

void ForecastContener::deleteObsolete(int hours)
{
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); )
	{
		if( (*it).loadTime.elapsed() > 60*60*1000)
			it = list.remove(it);
		else
			it++;
	}
}

void ForecastContener::add(const Forecast& forecast)
{	
	bool replace = false;
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); it++)
	{
		if( (*it).LocationID == forecast.LocationID && (*it).config == forecast.config)
		{
			(*it) = forecast;
			replace = true;
			break;
		}
	}
	
	if(!replace)
		list.push_front(forecast);
}

==> weather-2.02.tar.bz2: weather/weather_parser.cpp <==
/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "weather_parser.h"

WeatherParser::WeatherParser(QString& page, ConfigFile* cf)
:
	strPage(page),
	config(cf)
{
}

/**
	Formatowanie wyjciowej prognozy pogody
**/
bool WeatherParser::getData(ConfigFile* wConfig, Forecast& forecast, const QString& iconPath)
{
	kdebugf();

	bool cs = wConfig->readBoolEntry("Default","CaseSensitive");
	
	int num_layouts = wConfig->readNumEntry("Default","Layouts");
	WDataValue dValue;
	
	QString actualName;
	QValueList<WDataValue> valsList;
	QStringList nextDaysNamesList;
	QValueList<WDataValue> nextDaysValsList;
	
	bool result;
	int cursor = 0;
	for(int i=0; i<num_layouts; i++)
	{
		QString layoutSection = QString("Layout%1").arg(i+1);
		QString layoutName = wConfig->readEntry(layoutSection,"Name");
		int num_repeats = wConfig->readNumEntry(layoutSection,"Repeats");
		int num_values = wConfig->readNumEntry(layoutSection,"Values");
		
		for(int j=0; j<num_repeats; j++)
		{
			for(int k=0; k<num_values; k++)
			{
				QString valSection = QString("Layout%1Value%2").arg(i+1).arg(k+1);
				
				dValue.Name = wConfig->readEntry(valSection,"Name");
				dValue.Start = wConfig->readEntry(valSection,"Start");
				dValue.End = wConfig->readEntry(valSection,"End");
				dValue.Content = "";
				
				result = getDataValue(dValue,cursor, wConfig, cs);
				
				if(dValue.Content.isEmpty())
					continue;
				else if(dValue.Name == "Phantom")
					continue;
				
				if(layoutName == "Name")
					forecast.LocationName = dValue.Content;
				else if(layoutName == "Actual")
				{
					if(dValue.Name == "Name")
						actualName = dValue.Content;
					else
						valsList.push_back(dValue);
				}
				else if(layoutName == "DaysNames")
				{
					if(dValue.Name == "Actual")
						actualName = dValue.Content;
					else if(dValue.Name == "NextDays")
						nextDaysNamesList.push_back(dValue.Content);
				}
				else if(layoutName == "NextDays" && dValue.Name == "Name")
					nextDaysNamesList.push_back(dValue.Content);
				else
					nextDaysValsList.push_back(dValue);
			}
		}
	}
	
	QString Data("");
	
	ForecastDay fDay;
	
	fDay["Name"] = actualName;
	for(QValueList<WDataValue>::iterator it = valsList.begin(); it != valsList.end(); it++)
	{
		if((*it).Name == "Icon")
			fDay["Icon"] = iconPath + wConfig->readEntry("Icons",(*it).Content);
		else
			fDay[(*it).Name] = (*it).Content;
	}
	
	forecast.Days.push_back(fDay);
	fDay.clear();
	
	int num_vals = nextDaysValsList.count();
	int num_names = nextDaysNamesList.count();
	
	if(!num_names)
		return false;
		
	int T = num_vals/num_names;
	
	if(!T)
		return false;
	
	Data = "";
	for(int i=0; i<num_vals; i++)
	{
		WDataValue& val = nextDaysValsList[i];
	
		if(i%T == 0)
			fDay["Name"] = nextDaysNamesList[i/T];
		
		if(val.Name == "Icon")
			fDay["Icon"] = iconPath + wConfig->readEntry("Icons",val.Content);
		else
			fDay[val.Name] = val.Content;
		
		if(i%T == T-1)
		{
			forecast.Days.push_back(fDay);
			fDay.clear();
			
			Data = "";
		}
	}

	kdebugf2();
	return true;
}

/**
	Wyawianie fragmentu ze strony
**/
bool WeatherParser::getDataValue(WDataValue& wdata, int& cursor, ConfigFile* wConfig, bool CaseSensitive)
{
	kdebugf();
	
	long int start, end;
	long int startData;
	
	start = strPage.find(wdata.Start, cursor, CaseSensitive);
	if(start == -1)
		return false;
	
	startData = start + wdata.Start.length();
	
	end = strPage.find(wdata.End, startData, CaseSensitive);
	if(end == -1)
		return false;
	
	cursor = end;
	wdata.Content = tagClean(strPage.mid(startData, end-startData));
	
	kdebugf2();
	return true;
}

/**
	Zwraca wyszukane identyfikatory miast dla szukanego miasta
**/
void WeatherParser::getSearch(QStringList& names, QStringList& codes, ConfigFile* wConfig)
{
	kdebugf();
	
	bool CaseSensitive = wConfig->readBoolEntry("Default","CaseSensitive");

	long int start, separator, end, current;
	long int startData, sepData;
	bool idFirst;
	QString section, starttag, septag, endtag, first, second;
	
	int countResults = wConfig->readNumEntry("Name Search","SearchResults");
	
	for(int i=0; i<countResults; i++)
	{
		section = QString("SearchResult%1").arg(i+1);
		idFirst = wConfig->readBoolEntry(section,"IDFirst");
		starttag = wConfig->readEntry(section,"Start");
		septag = wConfig->readEntry(section,"Separator");
		endtag = wConfig->readEntry(section,"End");
	
		current = 0;
		do{
			start = strPage.find(starttag, current, CaseSensitive);	
			startData = start + starttag.length();
			separator = strPage.find(septag, startData, CaseSensitive);
			sepData = separator + septag.length();
			end = strPage.find(endtag, sepData, CaseSensitive);
			
			if(end != -1 && start != -1 && separator != -1)
			{
				first = strPage.mid(startData, separator-startData);
				second = strPage.mid(sepData, end-sepData);
				
				if(!first.isEmpty() && !second.isEmpty())
				{
					if(idFirst)
					{
						names.push_front(tagClean(second));
						codes.push_front(first);
					}
					else
					{
						names.push_front(tagClean(first));
						codes.push_front(second);
					}
				}
				current = end+endtag.length();
			}
		}while(end != -1 && start != -1 && separator != -1);
	}
	
	kdebugf2();
}

/**
	Wyowienie szukanego ID miasta z adresu URL.
	(niektre serwisy przekierowywuj od razu na
	stron z prognoz, gdy znajd tylko jedno miasto)
**/
QString WeatherParser::getFastSearch(const QString& link, const ConfigFile& wConfig)
{
	kdebugf();
	
	QString starttag, endtag;
	long int start, end, startData;
	
	starttag = wConfig.readEntry("Name Search","FastSearch Start");
	endtag = wConfig.readEntry("Name Search","FastSearch End");
	
	start = link.find(starttag, 0, false);
	startData = start + starttag.length();
	
	if(endtag.isEmpty())
		end = link.length();
	else
		end = link.find(endtag, startData, false);
	
	if(start == -1 || end == -1)
		return "";
	else
		return link.mid(startData, end-startData);

	kdebugf2();
}

/**
	Czyszczenie bufora z tagw HTML i zamienianie znakw specjalnych
**/
QString WeatherParser::tagClean(QString str)
{
	kdebugf();
	
	//str.replace("&deg;","");
	str.replace("&nbsp;"," ");
	
	int start, end;
	start = 0;
	do{
		start = str.find("<",start);
		end = str.find(">",start+1);
		
		if(start != -1 && end != -1)
			str.replace(start,end+1-start, " ");
			
	}while(start != -1 && end != -1);
	
	str.replace("\n"," ");
	str.replace("\r"," ");
	str.replace("  "," ");
	str.replace(" ,",",");
	str.replace(" .",".");
	str.replace(" :",":");
	str.replace(" / ","/");
	
	kdebugf2();
	return str;
}


/**
	ForecastContener
**/

bool ForecastContener::getForecast(const QString& configFile, const QString& location, Forecast& forecast, int hours)
{
	kdebugf();
	
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); it++)
	{
		if( (*it).LocationID == location && (*it).config == configFile)
		{
			if( (*it).loadTime.elapsed() > hours*60*60*1000 )
			{
				list.remove(it);
				return false;
			}
			else
			{
				forecast = (*it);
				return true;
			}
		}
	}
	
	kdebugf2();
	return false;
}

void ForecastContener::deleteObsolete(int hours)
{
	kdebugf();
	
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); )
	{
		if( (*it).loadTime.elapsed() > 60*60*1000)
			it = list.remove(it);
		else
			it++;
	}
	
	kdebugf2();
}

void ForecastContener::add(const Forecast& forecast)
{
	kdebugf();
	
	bool replace = false;
	for(QValueList<Forecast>::iterator it = list.begin(); it != list.end(); it++)
	{
		if( (*it).LocationID == forecast.LocationID && (*it).config == forecast.config)
		{
			(*it) = forecast;
			replace = true;
			break;
		}
	}
	
	if(!replace)
		list.push_front(forecast);
	
	kdebugf2();
}
